/* xdwapi sample 7

Function :
  Creates a new DocuWorks binder and inserts DocuWorks documents into the binder.
How to use :
  xdwsmpl7 binder.xbd [[document.xdw]...]

  binder.xbd : DocuWorks binder (out).
  document.xdw : DocuWorks document (in).

  The color and size of the binder are default values.
  If DocuWorks documents are omitted, creates an empty DocuWorks binder.
*/

#include <stdio.h>
#include <windows.h>

#include <xdw_api.h>

void print_error(int code)
{
	switch (code) {
	case XDW_E_NOT_INSTALLED:
		fprintf(stderr, "DocuWorks not installed.");
		break;
	case XDW_E_FILE_NOT_FOUND:
		fprintf(stderr, "File not found.");
		break;
	case XDW_E_ACCESSDENIED:
	case XDW_E_INVALID_NAME:
	case XDW_E_BAD_NETPATH:
		fprintf(stderr, "File not opened.");
		break;
	case XDW_E_BAD_FORMAT:
		fprintf(stderr, "File bad format.");
		break;
	case XDW_E_INVALID_ACCESS:
		fprintf(stderr, "Invalid access.");
		break;
	default:
		fprintf(stderr, "Error. codeF0x%X", code);
		break;
	}
}

int main(int argc, char** argv)
{
	int api_result = 0;

	if (argc < 2) {
		fprintf(stderr, "How to use : xdwsmpl7 binder.xbd [[document.xdw]...]");
		return 0;
	}

	char in_path[_MAX_PATH];
	_fullpath(in_path, argv[1], _MAX_PATH);

	// Create a new binder.
	api_result = XDW_CreateBinder(in_path, NULL, NULL);
	if (api_result < 0) {
		print_error(api_result);
		return 0;
	}

	// Open the document handle.
	XDW_DOCUMENT_HANDLE h = NULL;
	XDW_OPEN_MODE_EX mode = {
		sizeof(XDW_OPEN_MODE_EX), XDW_OPEN_UPDATE, XDW_AUTH_NODIALOGUE };
	api_result = XDW_OpenDocumentHandle(in_path, &h, (XDW_OPEN_MODE*)&mode);
	if (api_result < 0) {
		print_error(api_result);
		return 0;
	}

	// Insert the documents to the binder.
	int doc_num = argc - 2;
	for (int i = 1; i <= doc_num; i++) {
		_fullpath(in_path, argv[i + 1], _MAX_PATH);
		api_result = XDW_InsertDocumentToBinder(h, i, in_path, NULL);
		if (api_result < 0) {
			print_error(api_result);
			break;
		}
	}

	// Reflect the changes to the document.
	if (api_result >= 0) {
		api_result = XDW_SaveDocument(h, NULL);
	}

	// Close the document handle.
	XDW_CloseDocumentHandle(h, NULL);

	if (api_result < 0) return 0;

	return 1;
}
