/* xdwapi sample 11

Function :
    Attatches a signature to a DocuWorks file.
How to use :
    xdwsmpl11 input_file output_file signature_module
        signature_module : DocuWorks Electronic Signature=0 Electronic Certificate=1
*/

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using FujiXerox.DocuWorks.Toolkit;

namespace XdwSmpl11
{
    class Program
    {
        static int Main(string[] args)
        {
            if (args.Length != 3)
            {
                print_usage();
                return 0;
            }
            int sig_type = int.Parse(args[2]);
            if (sig_type != 0 && sig_type != 1)
            {
                print_usage();
                return 0;
            }

            Xdwapi.XDW_SIGNATURE_OPTION_V5 option = new Xdwapi.XDW_SIGNATURE_OPTION_V5();
            option.Page = 1; // first page
            option.HorPos = 1000; // 10mm from the left
            option.VerPos = 1000; // 10mm from the top
            if (sig_type == 0)
            {
                option.SignatureType = Xdwapi.XDW_SIGNATURE_STAMP;
            }
            else
            {
                option.SignatureType = Xdwapi.XDW_SIGNATURE_PKI;
            }

            // Sign.
            Xdwapi.XDW_SIGNATURE_MODULE_STATUS module_status = new Xdwapi.XDW_SIGNATURE_MODULE_STATUS();
            int api_result = Xdwapi.XDW_SignDocument(args[0], args[1], option, null, ref module_status);

            if (api_result < 0)
            {
                // Print XDWAPI error.
                print_error(api_result);
                if (api_result == Xdwapi.XDW_E_SIGNATURE_MODULE)
                {
                    // Print error unique to the signature module.
                    print_module_error(module_status.SignatureType, module_status.ErrorStatus);
                }
                return 0;
            }

            return 1;
        }

        // Print XDWAPI error.
        static void print_error(int code)
        {
            TextWriter errorWriter = Console.Error;
            switch (code)
            {
                case Xdwapi.XDW_E_NOT_INSTALLED:
                    errorWriter.WriteLine("DocuWorks not installed.");
                    break;
                case Xdwapi.XDW_E_FILE_NOT_FOUND:
                    errorWriter.WriteLine("File not found.");
                    break;
                case Xdwapi.XDW_E_ACCESSDENIED:
                case Xdwapi.XDW_E_INVALID_NAME:
                case Xdwapi.XDW_E_BAD_NETPATH:
                    errorWriter.WriteLine("File not opened.");
                    break;
                case Xdwapi.XDW_E_BAD_FORMAT:
                    errorWriter.WriteLine("File bad format.");
                    break;
                case Xdwapi.XDW_E_INVALID_ACCESS:
                    errorWriter.WriteLine("Invalid access.");
                    break;
                case Xdwapi.XDW_E_SIGNATURE_MODULE:
                    errorWriter.WriteLine("Error unique to the signature module occurred.");
                    break;
                default:
                    errorWriter.WriteLine("Error.");
                    break;
            }
        }

        // Print error unique to the signature module.
        static void print_module_error(int module, int code)
        {
            TextWriter errorWriter = Console.Error;
            switch (module)
            {
                // DocuWorks Electronic Signature
                case Xdwapi.XDW_SIGNATURE_STAMP:
                    errorWriter.WriteLine("DocuWorks Electronic Signature : ");
                    switch (code)
                    {
                        case Xdwapi.XDW_SIGNATURE_STAMP_ERROR_NO_OPENING_CASE:
                            errorWriter.WriteLine("The processing failed because no open Electronic Signature Case exists.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_STAMP_ERROR_NO_SELFSTAMP:
                            errorWriter.WriteLine("My Electronic Signature is not available in the Electronic Signature Case.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_STAMP_ERROR_OUT_OF_VALIDITY:
                            errorWriter.WriteLine("The Electronic Signature is outside the valid date.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_STAMP_ERROR_INVALID_DATA:
                            errorWriter.WriteLine("The processing failed because of uncomformity of data such as document to attach the signature and Electronic Signature Case.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_STAMP_ERROR_OUT_OF_MEMORY:
                            errorWriter.WriteLine("The processing failed due to insufficient memory.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_STAMP_ERROR_UNKNOWN:
                            errorWriter.WriteLine("The processing failed due to unexpected error.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_STAMP_ERROR_OK:
                            errorWriter.WriteLine("The processing succeeded.");
                            break;
                        default:
                            errorWriter.WriteLine("Error.");
                            break;
                    }
                    break;
                // Electronic Certificate
                case Xdwapi.XDW_SIGNATURE_PKI:
                    errorWriter.WriteLine("Electronic Certificate module : ");
                    switch (code)
                    {
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_BAD_PLATFORM:
                            errorWriter.WriteLine("Executing environment is inappropriate.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_WRITE_REG_ERROR:
                            errorWriter.WriteLine("Writing operation to the registry failed.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_BAD_TRUST_LEVEL:
                            errorWriter.WriteLine("The set trustworthiness level is invalid.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_BAD_REVOKE_CHECK_TYPE:
                            errorWriter.WriteLine("The set verification method is invalid.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_BAD_AUTO_IMPORT_CERT_FLAG:
                            errorWriter.WriteLine("The value of auto registration flag of the Certificate is invalid.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_BAD_SIGN_CONFIG:
                            errorWriter.WriteLine("The setting of the signature is invalid.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_NO_IMAGE_FILE:
                            errorWriter.WriteLine("Image file does not exist.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_BAD_SIGN_CERT:
                            errorWriter.WriteLine("The selected Certificate can no longer be used.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_NO_SIGN_CERT:
                            errorWriter.WriteLine("The Certificate for signature is not selected.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_NOT_USE_PRIVATE_KEY:
                            errorWriter.WriteLine("Access to Private key failed.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_INVALID:
                            errorWriter.WriteLine("The Certificate is already revoked.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_BAD_SIGN:
                            errorWriter.WriteLine("The signature of the Certificate is invalid.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_REVOKE_CHECK_ERROR:
                            errorWriter.WriteLine("Revocation of the Certificate cannot be confirmed.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_OUT_OF_VALIDITY:
                            errorWriter.WriteLine("The Certificate is outside the valid date.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_NO_CERT:
                            errorWriter.WriteLine("The Certificate does not exist.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_FAILURE_IMPOPT_CERT:
                            errorWriter.WriteLine("Registration of the Certificate failed.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_NO_ROOT_CERT:
                            errorWriter.WriteLine("The Root Certificate does not exist.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_BAD_CERT_SIZE:
                            errorWriter.WriteLine("The Certificate size is invalid.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_BAD_ARG:
                            errorWriter.WriteLine("The argument is invalid.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_UNKNOWN:
                            errorWriter.WriteLine("Unknown error.");
                            break;
                        case Xdwapi.XDW_SIGNATURE_PKI_ERROR_OK:
                            errorWriter.WriteLine("The processing succeeded.");
                            break;
                        default:
                            errorWriter.WriteLine("Error.");
                            break;
                    }
                    break;
                default:
                    errorWriter.WriteLine("Unknown signature module.");
                    break;
            }
        }

        // Print usage.
        static void print_usage()
        {
            TextWriter errorWriter = Console.Error;
            errorWriter.WriteLine("Usage : xdwsmpl11 input_file output_file signature_module");
            errorWriter.WriteLine("\tsignature_module : DocuWorks Electronic Signature=0 Digital Certificate=1");
        }
    }
}
