/* xdwapi sample 5

Function :
  Attaches the text annotations with date and hour at each page of the DocuWorks document.
How to use :
  xdwsmpl5 document

  document : DocuWorks document (in).
*/

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using FujiXerox.DocuWorks.Toolkit;

namespace XdwSmpl5
{
    class Program
    {
        static int Main(string[] args)
        {
            int api_result = 0;

            if (args.Length != 1) {
                TextWriter errorWriter = Console.Error;
                errorWriter.WriteLine("How to use : xdwsmpl5 document");
                return 0;
            }
            
            string in_path = Path.GetFullPath(args[0]);

            // Open the document handle.
            Xdwapi.XDW_DOCUMENT_HANDLE handle = new Xdwapi.XDW_DOCUMENT_HANDLE();
            Xdwapi.XDW_OPEN_MODE_EX mode = new Xdwapi.XDW_OPEN_MODE_EX();
            mode.Option = Xdwapi.XDW_OPEN_UPDATE;
            mode.AuthMode = Xdwapi.XDW_AUTH_NODIALOGUE;
            api_result = Xdwapi.XDW_OpenDocumentHandle(in_path, ref handle, mode);
            if (api_result < 0) {
                print_error(api_result);
                return 0;
            }

            // Use XDW_GetDocumentInformation to get the number of total pages.
            Xdwapi.XDW_DOCUMENT_INFO info = new Xdwapi.XDW_DOCUMENT_INFO();
            Xdwapi.XDW_GetDocumentInformation(handle, ref info);
            int last_page = info.Pages;

            // Get the current date and hour.
            DateTime current = System.DateTime.Now;
            string date_time = current.ToLongDateString() + " " + current.ToLongTimeString();

            // Attach an annotation at each page.
            for (int i = 1; i <= last_page; i++) {
                api_result = add_annotation(handle, i, date_time);
                if (api_result < 0) {
                    print_error(api_result);
                    break;
                }
            }

            // Reflect the changes to the document.
            if (api_result >= 0) {
                api_result = Xdwapi.XDW_SaveDocument(handle);
            }

            // Close the document handle.
            Xdwapi.XDW_CloseDocumentHandle(handle);

            if (api_result < 0)
                return 0;

            return 1;
        }

        static int add_annotation(Xdwapi.XDW_DOCUMENT_HANDLE handle, int page, string date_time)
        {
            // Attach a text annotation.
            Xdwapi.XDW_ANNOTATION_HANDLE annoation = new Xdwapi.XDW_ANNOTATION_HANDLE();
            int api_result = Xdwapi.XDW_AddAnnotation(
                handle, Xdwapi.XDW_AID_TEXT, page, 500, 500, null, ref annoation);
            if (api_result < 0)
                return api_result;

            // Set the string to the text annotation.
            api_result = Xdwapi.XDW_SetAnnotationAttribute(
                handle, annoation, Xdwapi.XDW_ATN_Text, Xdwapi.XDW_ATYPE_STRING, date_time);

            return api_result;
        }

        static void print_error(int code)
        {
            TextWriter errorWriter = Console.Error;
            switch (code) {
                case Xdwapi.XDW_E_NOT_INSTALLED:
                    errorWriter.WriteLine("DocuWorks not installed.");
                    break;
                case Xdwapi.XDW_E_FILE_NOT_FOUND:
                    errorWriter.WriteLine("File not found.");
                    break;
                case Xdwapi.XDW_E_ACCESSDENIED:
                case Xdwapi.XDW_E_INVALID_NAME:
                case Xdwapi.XDW_E_BAD_NETPATH:
                    errorWriter.WriteLine("File not opened.");
                    break;
                case Xdwapi.XDW_E_BAD_FORMAT:
                    errorWriter.WriteLine("File bad format.");
                    break;
                case Xdwapi.XDW_E_INVALID_ACCESS:
                    errorWriter.WriteLine("Invalid access.");
                    break;
                default:
                    errorWriter.WriteLine("Error.");
                    break;
            }
        }

    }
}
