/* xdwapi sample 6

Function :
  Automatically rotates each page of DocuWorks document and performs OCR.
How to use :
  xdwsmpl6 document

  document : DocuWorks document (in).

  The default operation of OCR is performed.
*/

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using FujiXerox.DocuWorks.Toolkit;

namespace XdwSmpl6
{
    class Program
    {
        [STAThread]
        static int Main(string[] args)
        {
            int api_result = 0;

            if (args.Length != 1) {
                TextWriter errorWriter = Console.Error;
                errorWriter.WriteLine("How to use : xdwsmpl6 document");
                return 0;
            }

            string in_path = Path.GetFullPath(args[0]);

            // Open the document handle.
            Xdwapi.XDW_DOCUMENT_HANDLE handle = new Xdwapi.XDW_DOCUMENT_HANDLE();
            Xdwapi.XDW_OPEN_MODE_EX mode = new Xdwapi.XDW_OPEN_MODE_EX();
            mode.Option = Xdwapi.XDW_OPEN_UPDATE;
            mode.AuthMode = Xdwapi.XDW_AUTH_NODIALOGUE;
            api_result = Xdwapi.XDW_OpenDocumentHandle(in_path, ref handle, mode);
            if (api_result < 0) {
                print_error(api_result);
                return 0;
            }

            // Use XDW_GetDocumentInformation to get the number of total pages.
            Xdwapi.XDW_DOCUMENT_INFO info = new Xdwapi.XDW_DOCUMENT_INFO();
            Xdwapi.XDW_GetDocumentInformation(handle, ref info);
            int last_page = info.Pages;
            
            // Automatically rotate each page and perform OCR.
            int is_modified = 0;
            for (int i = 1; i <= last_page; i++) {
                api_result = Xdwapi.XDW_RotatePageAuto(handle, i);
                if (api_result >= 0) {
                    is_modified = 1;
                }
                api_result = Xdwapi.XDW_ApplyOcr(handle, i, Xdwapi.XDW_OCR_ENGINE_DEFAULT, null);
                if (api_result >= 0) {
                    is_modified = 1;
                }
                if (api_result == Xdwapi.XDW_E_CANCELED)
                {
                    print_error(api_result);
                    break;
                }
            }

            // Reflect the changes to the document.
            if (is_modified == 1) {
                api_result = Xdwapi.XDW_SaveDocument(handle);
            }

            // Close the document handle.
            Xdwapi.XDW_CloseDocumentHandle(handle);

            // Terminate the process of XDW_ApplyOcr.
            Xdwapi.XDW_Finalize();

            if (api_result < 0)
                return 0;

            return 1;
        }

        static void print_error(int code)
        {
            TextWriter errorWriter = Console.Error;
            switch (code)
            {
                case Xdwapi.XDW_E_NOT_INSTALLED:
                    errorWriter.WriteLine("DocuWorks not installed.");
                    break;
                case Xdwapi.XDW_E_FILE_NOT_FOUND:
                    errorWriter.WriteLine("File not found.");
                    break;
                case Xdwapi.XDW_E_ACCESSDENIED:
                case Xdwapi.XDW_E_INVALID_NAME:
                case Xdwapi.XDW_E_BAD_NETPATH:
                    errorWriter.WriteLine("File not opened.");
                    break;
                case Xdwapi.XDW_E_BAD_FORMAT:
                    errorWriter.WriteLine("File bad format.");
                    break;
                case Xdwapi.XDW_E_INVALID_ACCESS:
                    errorWriter.WriteLine("Invalid access.");
                    break;
                case Xdwapi.XDW_E_CANCELED:
                    errorWriter.WriteLine("Canceled.");
                    break;
                default:
                    errorWriter.WriteLine("Error. code: 0x{0}", code.ToString("X"));
                    break;
            }
        }


    }
}
